@@ -1,185 +1,203 @@
 #' Convert to an ioslides Presentation
 #'
 #' Format for converting from R Markdown to an
-#' \href{https://code.google.com/archive/p/io-2012-slides/}{ioslides} presentation.
+#' [ioslides](https://code.google.com/archive/p/io-2012-slides/) presentation.
 #'
+#' @md
 #' @inheritParams html_document
 #' @param logo Path to file that includes a logo for use in the presentation
 #'   (should be square and at least 128x128).
 #' @param slide_level Header level to consider as slide separator (Defaults to
 #'   header 2).
-#' @param incremental \code{TRUE} to render slide bullets incrementally.
+#' @param incremental `TRUE` to render slide bullets incrementally.
 #'   Note that if you want to reverse the default incremental behavior for an
-#'   individual bullet you can preceded it with \code{>}.
-#'   For example: \emph{\code{> - Bullet Text}}.
+#'   individual bullet you can preceded it with `>`.
+#'   For example: *`> - Bullet Text`*.
 #' @param widescreen Display presentation with wider dimensions.
 #' @param smaller Use smaller text on all slides. You can also enable this for
-#'   individual slides by adding the \code{.smaller} attribute to the slide
-#'   header (see \emph{Presentation Size} below for details).
+#'   individual slides by adding the `.smaller` attribute to the slide
+#'   header (see *Presentation Size* below for details).
 #' @param transition Speed of slide transitions. This can be "default",
 #'   "slower", "faster", or a numeric value with a number of seconds (e.g. 0.5).
 #' @param analytics A Google analytics property ID.
 #' @param smart Produce typographically correct output, converting straight
-#'  quotes to curly quotes, \code{---} to em-dashes, \code{--} to en-dashes, and
-#'  \code{...} to ellipses.
+#'  quotes to curly quotes, `---` to em-dashes, `--` to en-dashes, and
+#'  `...` to ellipses.
 #' @param css One or more css files to include.
-#' @return R Markdown output format to pass to \code{\link{render}}.
+#' @return R Markdown output format to pass to [render()].
 #' @details
-#'   See the \href{https://bookdown.org/yihui/rmarkdown/ioslides-presentation.html}{
-#'   online documentation} for additional details on using the
-#'   \code{ioslides_presentation} format.
+#'   See the [
+#'   online documentation](https://bookdown.org/yihui/rmarkdown/ioslides-presentation.html) for additional details on using the
+#'   `ioslides_presentation` format.
 #'
-#'   Note that, if a \code{before_body} include is specified in \code{includes},
+#'   Note that, if a `before_body` include is specified in `includes`,
 #'   then it will replace the standard title slide entirely.
+#'
+#'   Regarding previewing slide in RStudio IDE, `ioslides_presentation()` will
+#'   always open preview in a new Window and the RStudio IDE configuration "Open in Viewer
+#'   Pane" will have no effect for this format.
+#'
 #' @section Slide Basics:
 #'   You can create a slide show broken up into sections by using the # and ##
 #'   heading tags (you can also create a new slide without a header using a
-#'   horizontal rule (\code{----------}). For example here's a simple slide show:
-#'   \preformatted{
-#' ---
-#' title: "Habits"
-#' author: John Doe
-#' date: March 22, 2005
-#' output: ioslides_presentation
-#' ---
+#'   horizontal rule (`----------`). For example here's a simple slide show:
+#'   ````markdown
+#'   ---
+#'   title: "Habits"
+#'   author: John Doe
+#'   date: March 22, 2005
+#'   output: ioslides_presentation
+#'   ---
 #'
-#' # In the morning
+#'   # In the morning
 #'
-#' ## Getting up
+#'   ## Getting up
 #'
-#' - Turn off alarm
-#' - Get out of bed
+#'   - Turn off alarm
+#'   - Get out of bed
 #'
-#' ## Breakfast
+#'   ## Breakfast
 #'
-#' - Eat eggs
-#' - Drink coffee
+#'   - Eat eggs
+#'   - Drink coffee
 #'
-#' # In the evening
+#'   # In the evening
 #'
-#' ## Dinner
+#'   ## Dinner
 #'
-#' - Eat spaghetti
-#' - Drink wine
+#'   - Eat spaghetti
+#'   - Drink wine
 #'
-#' ----------
+#'   ----------
 #'
-#' ![picture of spaghetti](images/spaghetti.jpg)
+#'   ![picture of spaghetti](images/spaghetti.jpg)
 #'
-#' ## Going to sleep
+#'   ## Going to sleep
 #'
-#' - Get in bed
-#' - Count sheep
+#'   - Get in bed
+#'   - Count sheep
+#'  ````
 #'
-#' }
 #'   You can add a subtitle to a slide or section by including text after the
 #'   pipe (|) character. For example:
-#'   \preformatted{## Getting up | What I like to do first thing}
+#'   ```markdown
+#'   ## Getting up | What I like to do first thing
+#'   ```
+#'
 #' @section Display Modes:
 #'   The following single character keyboard shortcuts enable alternate display
 #'   modes:
 #'   \itemize{
-#'     \item{\code{'f'}      }{enable fullscreen mode}
-#'     \item{\code{'w'}      }{toggle widescreen mode}
-#'     \item{\code{'o'}      }{enable overview mode}
-#'     \item{\code{'h'}      }{enable code highlight mode}
-#'     \item{\code{'p'}      }{show presenter notes}
+#'     \item{`'f'`      }{enable fullscreen mode}
+#'     \item{`'w'`      }{toggle widescreen mode}
+#'     \item{`'o'`      }{enable overview mode}
+#'     \item{`'h'`      }{enable code highlight mode}
+#'     \item{`'p'`      }{show presenter notes}
 #'   }
-#'   Pressing \code{Esc} exits all of these modes. See the sections below on
-#'   \emph{Code Highlighting} and \emph{Presenter Mode} for additional
+#'   Pressing `Esc` exits all of these modes. See the sections below on
+#'   *Code Highlighting* and *Presenter Mode* for additional
 #'   detail on those modes.
 #'
 #' @section Incremental Bullets:
-#'   You can render bullets incrementally by adding the \code{incremental}
+#'   You can render bullets incrementally by adding the `incremental`
 #'   option:
-#'   \preformatted{
-#' ---
-#' output:
-#'   ioslides_presentation:
-#'     incremental: true
-#' ---
-#' }
+#'   ```markdown
+#'   ---
+#'   output:
+#'     ioslides_presentation:
+#'       incremental: true
+#'   ---
+#'   ```
 #'   If you want to render bullets incrementally for some slides but not
 #'   others you can use this syntax:
-#'   \preformatted{
-#' > - Eat eggs
-#' > - Drink coffee
-#' }
+#'   ```
+#'   > - Eat eggs
+#'   > - Drink coffee
+#'   ```
+#'
 #' @section Presentation Size:
 #'   You can display the presentation using a wider form factor using the
-#'   \code{widescreen} option. You can specify that smaller text be used with
-#'   the \code{smaller} option. For example:
-#'   \preformatted{
-#' ---
-#' output:
-#'   ioslides_presentation:
-#'     widescreen: true
-#'     smaller: true
-#' ---
-#' }
-#'   You can also enable the \code{smaller} option on a slide-by-slide basis
-#'   by adding the \code{.smaller} attibute to the slide header:
-#'   \preformatted{## Getting up {.smaller}}
+#'   `widescreen` option. You can specify that smaller text be used with
+#'   the `smaller` option. For example:
+#'   ````yaml
+#'   ---
+#'   output:
+#'     ioslides_presentation:
+#'       widescreen: true
+#'       smaller: true
+#'   ---
+#'   ````
+#'   You can also enable the `smaller` option on a slide-by-slide basis
+#'   by adding the `.smaller` attibute to the slide header:
+#'   ```markdown
+#'   ## Getting up {.smaller}
+#'   ```
 #' @section Adding a Logo:
-#'   You can add a logo to the presentation using the \code{logo} option (the
+#'   You can add a logo to the presentation using the `logo` option (the
 #'   logo should be square and at least 128x128). For example:
-#'   \preformatted{
-#' ---
-#' output:
-#'   ioslides_presentation:
-#'     logo: logo.png
-#' ---
-#' }
+#'   ```markdown
+#'   ---
+#'   output:
+#'     ioslides_presentation:
+#'       logo: logo.png
+#'   ---
+#'   ```
 #'   A 128x128 version of the logo graphic will be added to the title slide and
 #'   an icon version of the logo will be included in the bottom-left footer of
 #'   each slide.
 #' @section Build Slides:
-#'   Slides can also have a \code{.build} attribute that indicate that their
+#'   Slides can also have a `.build` attribute that indicate that their
 #'   content should be displayed incrementally. For example:
-#'   \preformatted{## Getting up {.build}}
+#'   ```markdown
+#'   ## Getting up {.build}
+#'   ```
 #'   Slide attributes can be combined if you need to specify more than one,
 #'   for example:
-#'   \preformatted{## Getting up {.smaller .build}}
+#'   ```markdown
+#'   ## Getting up {.smaller .build}
+#'   ```
 #' @section Code Highlighting:
 #'   It's possible to select subsets of code for additional emphasis by adding a
 #'   special "highlight" comment around the code. For example:
-#'   \preformatted{
-#' ### <b>
-#' x <- 10
-#' y <- x * 2
-#' ### </b>
-#' }
+#'   ```markdown
+#'   ### <b>
+#'   x <- 10
+#'   y <- x * 2
+#'   ### </b>
+#'   ```
 #'   The highlighted region will be displayed with a bold font. When you want to
 #'   help the audience focus exclusively on the highlighted region press the
-#'   \code{'h'} key and the rest of the code will fade away.
+#'   `'h'` key and the rest of the code will fade away.
 #' @section Tables:
 #'   The ioslides template has an attractive default style for tables so you
 #'   shouldn't hesitate to add tables for presenting more complex sets of
 #'   information. Pandoc markdown supports several syntaxes for defining
 #'   tables which are described in the
-#'   \href{https://pandoc.org/MANUAL.html}{pandoc online documentation}.
+#'   [pandoc online documentation](https://pandoc.org/MANUAL.html).
 #' @section Advanced Layout:
-#'   You can center content on a slide by adding the \code{.flexbox}
-#'   and \code{.vcenter} attributes to the slide title. For example:
-#'   \preformatted{## Dinner {.flexbox .vcenter}}
-#'   You can horizontally center content by enclosing it in a \code{div} tag
-#'   with class \code{centered}. For example:
-#'   \preformatted{
-#' <div class="centered">
-#' This text is centered.
-#' </div>
-#' }
-#'   You can do a two-column layout using the \code{columns-2} class.
+#'   You can center content on a slide by adding the `.flexbox`
+#'   and `.vcenter` attributes to the slide title. For example:
+#'   ```markdown
+#'   ## Dinner {.flexbox .vcenter}
+#'   ```
+#'   You can horizontally center content by enclosing it in a `div` tag
+#'   with class `centered`. For example:
+#'   ```html
+#'   <div class="centered">
+#'   This text is centered.
+#'   </div>
+#'   ```
+#'   You can do a two-column layout using the `columns-2` class.
 #'   For example:
-#'   \preformatted{
-#' <div class="columns-2">
-#'   ![Image](image.png)
+#'   ```
+#'   <div class="columns-2">
+#'     ![Image](image.png)
 #'
-#'   - Bullet 1
-#'   - Bullet 2
-#'   - Bullet 3
-#' </div>
-#' }
+#'     - Bullet 1
+#'     - Bullet 2
+#'     - Bullet 3
+#'   </div>
+#'   ```
 #'   Note that content will flow across the columns so if you want to
 #'   have an image on one side and text on the other you should make
 #'   sure that the image has sufficient height to force the text to
@@ -188,33 +206,37 @@
 #'   You can color content using base color classes red, blue, green, yellow,
 #'   and gray (or variations of them e.g. red2, red3, blue2, blue3, etc.).
 #'   For example:
-#'   \preformatted{
-#' <div class="red2">
-#' This text is red
-#' </div>
-#' }
+#'   ```html
+#'   <div class="red2">
+#'   This text is red
+#'   </div>
+#'   ```
 #' @section Presenter Mode:
 #'   A separate presenter window can also be opened (ideal for when you are
 #'   presenting on one screen but have another screen that's private to you).
 #'   The window stays in sync with the main presentation window and also
 #'   shows presenter notes and a thumbnail of the next slide. To enable
-#'   presenter mode add \code{?presentme=true} to the URL of the presentation,
+#'   presenter mode add `?presentme=true` to the URL of the presentation,
 #'   for example:
-#'   \preformatted{mypresentation.html?presentme=true}
+#'   ```
+#'   mypresentation.html?presentme=true
+#'   ```
 #'   The presenter mode window will open and will always re-open with the
 #'   presentation until it's disabled with:
-#'   \preformatted{mypresentation.html?presentme=false}
+#'   ```
+#'   mypresentation.html?presentme=false
+#'   ```
 #'   To add presenter notes to a slide you include it within a "notes"
-#'   \code{div}. For example:
-#'   \preformatted{
-#' <div class="notes">
-#' This is my *note*.
+#'   `div`. For example:
+#'   ```
+#'   <div class="notes">
+#'   This is my *note*.
 #'
-#' - It can contain markdown
-#' - like this list
+#'   - It can contain markdown
+#'   - like this list
 #'
-#' </div>
-#' }
+#'   </div>
+#'   ```
 #' @section Printing and PDF Output:
 #'   You can print an ioslides presentation from within browsers that have
 #'   good support for print CSS (i.e. as of this writing Google Chrome
@@ -239,6 +261,7 @@ ioslides_presentation <- function(number_sections = FALSE,
                                   widescreen = FALSE,
                                   smaller = FALSE,
                                   transition = "default",
+                                  math_method = "mathjax",
                                   mathjax = "default",
                                   analytics = NULL,
                                   template = NULL,
@@ -254,6 +277,13 @@ ioslides_presentation <- function(number_sections = FALSE,
   # base pandoc options for all output
   args <- c()
 
+  # math
+  math <- mathjax_to_math(mathjax, math_method)
+  math <- check_math_argument(math)
+  if (!identical(math$engine, "mathjax")) {
+    stop2("Only mathjax is supported for `ioslide_presentation()` for 'math'.")
+  }
+
   # widescreen
   if (widescreen)
     args <- c(args, "--variable", "widescreen");
