@@ -47,11 +47,11 @@ pandoc_output_ext <- function(ext, to, input) {
 # From pkgdown:
 # https://github.com/r-lib/pkgdown/blob/04d3a76892320ac4bd918b39604c157e9f83507a/R/utils-fs.R#L85
 pkg_file <- function(..., package = "rmarkdown", mustWork = FALSE) {
-  if (is.null(devtools_meta(package))) {
-    system.file(..., package = package, mustWork = mustWork)
-  } else {
+  if (devtools_loaded(package)) {
     # used only if package has been loaded with devtools or pkgload
-    file.path(getNamespaceInfo(package, "path"), "inst", ...)
+    file.path(find.package(package), "inst", ...)
+  } else {
+    system.file(..., package = package, mustWork = mustWork)
   }
 }
 
@@ -89,6 +89,10 @@ pkg_file_lua <- function(filters = NULL, package = "rmarkdown") {
   pandoc_path_arg(files)
 }
 
+pkg_file_highlight <- function(file) {
+  pkg_file("rmarkdown", "highlight", file, mustWork = TRUE)
+}
+
 #' @rdname rmarkdown_format
 #' @export
 from_rmarkdown <- function(implicit_figures = TRUE, extensions = NULL) {
@@ -108,6 +112,15 @@ is_null_or_string <- function(text) {
   is.null(text) || (is.character(text) && (length(text) == 1))
 }
 
+# TODO: Export knitr:::is_blank() in xfun, and use here
+is_blank <- function(x) {
+  if (length(x)) {
+    all(grepl("^\\s*$", x))
+  } else {
+    TRUE
+  }
+}
+
 read_utf8 <- function(file) {
   if (inherits(file, 'connection')) con <- file else {
     con <- base::file(file, encoding = 'UTF-8'); on.exit(close(con), add = TRUE)
@@ -566,10 +579,13 @@ stop2 = function(...) stop(..., call. = FALSE)
 
 # devtools metadata -------------------------------------------------------
 
-# from pkgdown
+# from pkgdown & downlit
 # https://github.com/r-lib/pkgdown/blob/77f909b0138a1d7191ad9bb3cf95e78d8e8d93b9/R/utils.r#L52
 
-devtools_meta <- function(package) {
-  ns <- .getNamespace(package)
-  ns[[".__DEVTOOLS__"]]
+devtools_loaded <- function(x) {
+  if (!x %in% loadedNamespaces()) {
+    return(FALSE)
+  }
+  ns <- .getNamespace(x)
+  !is.null(ns$.__DEVTOOLS__)
 }
